/*
   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    Downloaded From www.ChromationSystems.com/rgbdemo.html  
    
    This is meant to be used with ColorMotion compatible firmware. Either 
    the Demo version or the Full version.
    
    Use this source to tweak or create your own software to interact with compatible
    controllers.    
*/


import processing.serial.*;

//variables for incomingdata
int inByte;
int inByte1;
int inByte2;

//variables for outgoing commands
int CmdByte=0;
int DataByte1=0;
int DataByte2=0;

int RedPWM[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int GreenPWM[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int BluePWM[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int Sequence[] = {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
int RunNum=0;
int LEDnum=0;

byte DataArray[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};


int SelectedButton=0;

int tempR=0; //hold registers
int tempG=0;
int tempB=0;


int LEDTotal=8; //this is changed in SetupHardware();

int LEDCounter=0;

//Hardware Info
String DeviceName="None";
String DeviceFirmware="None";
int HardwareID=0;
int FirmwareV=0;

String HWID[]= {
  "24 Chan. H.C. RGB", "24 Chan. H.C. Mono", "16 Channel RGB", "48 Channel Mono"
};
String FWV[]= {
  "Demo Firmware", "Firmware v.1", "Firmware v.2", "Firmware v.3", "Firmware v.4", "Firmware v.5", "Firmware v.6",
  "Firmware v.7", "Firmware v.8", "Firmware v.9", "Firmware v.10", "Incompatable Version"
};

//used for device communication handling
boolean LiveModeSent=false;
boolean RequestSent=false;
boolean ConfirmedFlag=false;
boolean StreamSend=false;
boolean DeviceConnected=true;
boolean ConnectionRequestSent=false; 

boolean GlobalDragging=false; //required global flag for Slider Bar
int SelectedSlider=0;//required global variable

boolean CascadeFlag = false; //used for LiveMode Cascade function
boolean GradientOpen=false; //for use with gradient color selection


//colors are used for the sliders, makes it easier to do it here than inline
color SliderBG = color(255);
color SliderHandCol = color(0);
color SliderSelCol = color(100);
color SliderStroke = color(0);

color CalcColor;

String portName;
Serial myPort;  

PImage backgroundImg;
PImage gradientImg;
PImage markImg;

SliderBar[] Sliders;
Button[] Buttons;

void setup()
{
  size(640, 480);
  background(127);

  frameRate(100); //tested upto 50, it eventually lost position

  backgroundImg = loadImage("bg.png");
  gradientImg = loadImage("gradient.png");
  markImg  = loadImage("mark.png"); 
  image(backgroundImg,0,0);

  //trys to open the second com port, if not found defaults to first COM
  //if computer has onboard COM and the PIC USB device, it will default to
  // the USB device rather than onboard COM

  try { 
    portName = Serial.list()[1];
    println("Trying COM2");
  } 
  catch (ArrayIndexOutOfBoundsException e) {

    try {
      portName = Serial.list()[0];
    }
    catch (ArrayIndexOutOfBoundsException a) {
      portName = null;
    }
  }

  
  myPort = new Serial(this, portName, 115200); 
  println(portName);
  
  // END PORT SETUP
  Buttons = new Button[5];
  
  Buttons[0] = new Button("  LiveMode", 545, 5, 85, 20, true, false); //Live Mode
  Buttons[1] = new Button(" Color Cycle", 85, 35, 85, 20, true, false); //Color Cycle 
  Buttons[2] = new Button(" Open Gradient", 180, 35, 100, 20, true, false); //Open Gradient
  Buttons[3] = new Button("Setting Down", 290, 35, 85, 20, false, false); //Slide Down
  Buttons[4] = new Button("  Setting Up", 380, 35, 85, 20, false, false); //Slide Up
//start sliders

   Sliders = new SliderBar[16];
   //x position - y position - width - height - starting Value - Minimum - Maximum - Background Color - 
   //      Slider Handle Color - Selected Color - Stroke Color - CircleHandle - Dragging 
 
 for(int i=0;i != Sliders.length;i++)
      { 
      if(i < 8) 
      Sliders[i] = new SliderBar(90, 75+(i*40), 200,  10,  0, 0, 1536,SliderBG,SliderHandCol, SliderSelCol, SliderStroke, true,false);
      else //this half the sliders is for 16 RGB channel Controller
      Sliders[i] = new SliderBar(380, 75+((i-8)*40), 200,  10,  0, 0, 1536,SliderBG,SliderHandCol, SliderSelCol, SliderStroke, true,false);

       //draws the sliders and colored circles, otherwise its only updated when a slider is altered
         Sliders[i].display();     
         SelectedSlider=i;  
              
         CalculateColor(); //figures color based on value
         fill(CalcColor);
         stroke(0);
         strokeWeight(2);
      if(i < 8) 
      ellipse(60, 80+(i*40), 25, 25); 
      else ellipse(350, 80+((i-8)*40), 25, 25); 
    }


    for(int i=0;i != Buttons.length;i++)Buttons[i].display();

     
  //sends connection request to device, it responds with its hardware version and firmware version
  CmdByte=4; //Command 4 is device connection and partial software reset
  DataByte1=0; //no databytes
  DataByte2=0; //no databytes
  SendRequest(); //sends request to device
  ConnectionRequestSent=true;  //sets flag that will be checked later  
}


void serialEvent(Serial myPort)
{
  //Commands are sent to the device as follows:

  //host sends to device -> q -> Q -> z -> device sends N -> host responds with G -> device sends n -> Host sends -> Command -> data1 -> data2

  //see Readme for Command Bytes and info.

  println("Recieved: "+(char)inByte+" : "+inByte); //displays recieved character in character format and decimal


    inByte2=inByte1;
    inByte1=inByte;
    inByte = myPort.read();

    if (inByte=='N' && RequestSent==true) //acknowledged recieved, check if it was requested
    {
      //recieved acknowledge, exchange confirm
      myPort.write('G');
      ConfirmedFlag=true;
    }
    else if (inByte=='n' && ConfirmedFlag==true)  //if 'n' is recived, device is ready for command
    {
      //command request confirmed, send command data
      RequestSent=false; //clear request flag
      ConfirmedFlag=false;//clear confirmed flag

      //send the command data
      myPort.write(CmdByte); //these are all set before hand.
      myPort.write(DataByte1);
      myPort.write(DataByte2);
    } //end else if
    else if(ConnectionRequestSent==true)
    {
      //character exchange sucessful and command bytes have been sent
      //the device then sends 2 bytes, HardwareID -> FirmwareVersion      
      if(HardwareID==0) HardwareID=inByte;
      else 
      {
      FirmwareV=inByte;
      ConnectionRequestSent=false;
      SetupHardware();//figures LEDTotal, and sets display values
      RefreshScreen(); //displays info
      }
    }//end else   
} //end function



void draw() 
{

  if (StreamSend==true)
  {
    RunNum++;//keeps track of amount of runs
    ColorCycle(); //function cycles the colors
    println("Red: "+RedPWM[LEDnum]+" Green: "+GreenPWM[LEDnum]+" Blue: "+BluePWM[LEDnum]+" Seq: "+Sequence[LEDnum]+" Run#: "+RunNum);  
    SendLiveData();
  }

  //this is an elseif, because sliders shouldn't be altered during Color Cycling
  else if(GlobalDragging==true) //only check sliders if one is being dragged
  {
    //only update the slider being dragged
        if(Sliders[SelectedSlider].Dragging==true) //only display while Dragging is set
        { 
         Sliders[SelectedSlider].move(); 
         Sliders[SelectedSlider].display();                 
                 
         //this displays the color circle next to the sliders

         CalculateColor(); //takes the slider value and changes it to a color()
       
         fill(CalcColor);
         stroke(0);
         strokeWeight(2);
         textSize(10);
         
        if(SelectedSlider < 8) 
        ellipse(60, 80+(SelectedSlider*40), 25, 25);
        else 
        ellipse(350, 80+((SelectedSlider-8)*40), 25, 25);         
        noStroke();
        
        //move data from CalculateColor to PWM arrays
        RedPWM[SelectedSlider] = tempR;
        GreenPWM[SelectedSlider] = tempG;
        BluePWM[SelectedSlider] = tempB; 
        
        SendLiveData();
        }     
  } 
} //end draw


void SendLiveData()
{
  //sends the data byte by byte, Processing adds a small delay between sending bytes
  // for whatever reason.
  //Note, it can be faster to move all the data into a byte[] array, then use myPort.write(ByteArray)
  //the data is sent, as fast as possible, which is at least 10 fold faster than the method below.
  //currently the device can not handle data that fast, so leave it as it is.
 
     for (int i=0; i != LEDTotal; i++) myPort.write(RedPWM[i]);
     for (int i=0; i != LEDTotal; i++) myPort.write(GreenPWM[i]); 
     for (int i=0; i != LEDTotal; i++) myPort.write(BluePWM[i]);  
}
  
  

void CalculateColor()
{
 //Converts the 0 - 1536 Slider value into color values  
 // 0  Black(off) -> White -> Red -> Green -> Blue -> Purple   1536
 
int tempVal = Sliders[SelectedSlider].Value;

//println(tempVal);

if(tempVal < 255)
{ 
  tempB = tempVal;
  tempR = tempVal; 
  tempG = tempVal;   
}
else if(tempVal > 255 && tempVal < 512)
{
  tempR = 255;
  tempG = tempVal - 255;
  tempB = 0;
}
else if(tempVal > 512 && tempVal < 768)
{
  tempG = 255;
  tempR = 255 - (tempVal-512);
  tempB = 0;
}
else if(tempVal > 768 && tempVal < 1024)
{
  tempB = (tempVal-768);
  tempG = 255;
  tempR=0;
}

else if(tempVal >= 1024 && tempVal <= 1280)
{
  tempB = 255;
  tempG = 255 - (tempVal-1024);
  tempR=0;
}
else if(tempVal > 1280)
{
  tempG=0;
  tempB = 255;
  tempR = (tempVal-1281);
}

CalcColor = color(tempR, tempG, tempB);
}//end function



void ColorCycle()
{
  
//R->Y->G->T->B->W 
  
//must start with Sequence=1, RedPWM=255, GreenPWM=0, BluePWM=0
  
//this is one of many ways to cycle RGB colors, but this is preferred
//as it is creates the most vibrant colors but the overall brightness varies.

  for(int i=0; i != LEDTotal; i++)
  {
    LEDnum=i;
  
  switch(Sequence[LEDnum])
  {
  case 0: //white to red
    GreenPWM[LEDnum]--;    
    if(BluePWM[LEDnum] != 0) BluePWM[LEDnum]--;   
    if(GreenPWM[LEDnum]==0) Sequence[LEDnum]++;
    break;
  case 1: //red to yellow
    GreenPWM[LEDnum]++;
    if(GreenPWM[LEDnum]==255) Sequence[LEDnum]++;
    break;
  case 2://yellow to green
    RedPWM[LEDnum]--;
    if(RedPWM[LEDnum]==0) Sequence[LEDnum]++;
    break; 
  case 3: //green to teal
    BluePWM[LEDnum]++;
    if(BluePWM[LEDnum]==255) Sequence[LEDnum]++;
    break;  
  case 4: //teal to blue
    GreenPWM[LEDnum]--;
    if(GreenPWM[LEDnum]==0) Sequence[LEDnum]++;
    break;
  case 5: //blue to purple
    RedPWM[LEDnum]++;
    if(RedPWM[LEDnum]==255)Sequence[LEDnum]++;   
    break;    
  case 6: //blue to white
    GreenPWM[LEDnum]++;
    if(RedPWM[LEDnum]==255) Sequence[LEDnum] = 0;   
    break;
    }     
  }    
  }//end ColorCycle()

  
  




